/***************************************************************************
 *
 * Copyright (c) 1999 BalaBit Computing
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Inspired by nsyslog, originally written by Darren Reed.
 *
 * $Id: affile.c,v 1.13 1999/05/04 18:49:16 bazsi Exp $
 *
 ***************************************************************************/

#include "affile.h"
#include "format.h"
#include "pkt_buffer.h"
#include "cfgfile.h"

#include <fcntl.h>
#include <assert.h>
#include <errno.h>
#include <string.h>

#include "affile.c.x"

static int do_open_file(struct ol_string *name, int flags, int mode, int *fd)
{
	*fd = open(name->data, flags, mode);
	if (*fd != -1) {
		return 1;
	}
	return 0;
}

/* CLASS:
     (class 
       (name affile_source)
       (super log_source_driver)
       (vars
         (backend object io_backend)
         (src object io_fd)
         (name string)))
*/


static int do_init_affile_source(struct log_handler *c, struct syslog_conf *cfg)
{
	CAST(affile_source, self, c);
	int fd;
  
	if (do_open_file(self->name, O_RDONLY | O_NOCTTY, 0, &fd)) {
		
		self->src = io_read(self->backend, fd, 
			make_log_reader(0, c), 
			NULL);
		REMEMBER_RESOURCE(cfg->resources, &self->src->super.super);
		return ST_OK | ST_GOON;
	}
	else {
		werror("Error opening file %S for reading (%z)\n", self->name, strerror(errno));
	}
	return ST_FAIL | ST_QUIT;
}

static void do_affile_handle_line(struct log_handler *c, struct log_info *log)
{
	CAST(affile_source, self, c);
	log->flags |= LF_LOCAL;
	HANDLE_LOG(self->super.super.next, log);
}

struct log_source_driver *make_affile_source(const char *name, struct io_backend *backend)
{
	NEW(affile_source, self);
  
	self->super.super.super.init = do_init_affile_source;
	self->super.super.super.handler = do_affile_handle_line;
	self->backend = backend;
	self->name = c_format_cstring("%z%c", name, 0);
	return &self->super;
}

/* CLASS:
     (class
       (name affile_dest)
       (super log_dest_driver)
       (vars
         (name string)
         (flags simple int)
         (dest object io_fd)
	 (sync_freq simple UINT32)
         (backend object io_backend)))
*/

static int do_init_affile_dest(struct log_handler *c, struct syslog_conf *cfg)
{
	CAST(affile_dest, self, c);
	int fd;
	
	if (self->sync_freq == -1)
		self->sync_freq = cfg->sync_freq;
	if (do_open_file(self->name, O_WRONLY | O_CREAT | O_APPEND | O_NOCTTY, 0600, &fd)) {
		self->dest = io_write(self->backend, fd, 
			make_pkt_buffer_ext(self->sync_freq, MAX(LOG_FIFO_SIZE + 1, self->sync_freq)),
			NULL);
		self->super.writer = (struct abstract_write *) self->dest->buffer;
		REMEMBER_RESOURCE(cfg->resources, &self->dest->super.super);
		return ST_OK | ST_GOON;
	}
	else {
		werror("Cannot open file %S for writing (%z)\n", self->name, strerror(errno));
	}
	return ST_FAIL | ST_QUIT;
}

static void do_handle_affile_log(struct log_handler *c, struct log_info *msg)
{
	CAST(affile_dest, self, c);

	A_WRITE_STRING(self->super.writer, c_format("%S %S %S\n", msg->date, msg->host, msg->msg));
	log_info_free(msg);
}

struct log_dest_driver *make_affile_dest(const char *name, int flags, struct io_backend *backend)
{
	NEW(affile_dest, self);
	
#if 0
	init_dest_driver(&self->super);
#endif
	
	self->super.super.init = do_init_affile_dest;
	self->super.super.handler = do_handle_affile_log;
	self->backend = backend;
	self->name = c_format("%z%c", name, 0);
	self->name->length--;
	self->flags = flags;
	self->sync_freq = -1; /* use global setting */
	return &self->super;
}
